function [error conf] = main_call_file_ADNI(types, FEATURES, KERNELS)
%EXAMPLE: a group of points
%POINT: a fixed-length feature vector
%
%input:
%   types is a cell array with strings which name the different imaging
%       modalities, e.g., FDG (FluoroDeoxy-Glucose PET) and GM (sMRI Gram Matter maps).
%   FEATURES allows a subset of kernels to be used (to reduce running time)
%       based on how many features were used in building the kernel.
%   KERNELS is similar to FEATURES, except that it selects kernels types.
%   S gives 30 different permutations of the training examples, allowing
%       for 30x repeated cross-validation.
%output:
%   error is the averaged test set error rate.
%   conf is the test-set predictions for each fold / iteration. Can be
%       useful for computing ROCs, etc.


%Number of "EXAMPLES" per training fold. In this case, it means the number
%of samples of the ratio cut objective taken per kernel.
num_examples=4;
FOLDS = 10;
truth = actually_load('ground_truth.mat', 'truth');
M = numel(truth);

if nargin < 2
    for t = 1:numel(types)
        FEATURES{t} = 1:8;
        KERNELS{t} =  [1 2 5]; % 1:Linear, 2:3 Poly, 4:5 Gaussian
    end
end

num_kernels = 0;
for t=1:numel(types)
    num_kernels = num_kernels + numel(FEATURES{t}) * numel(KERNELS{t});
end

S = actually_load('S.mat', 'S');

%DEBUGGING STUFF
SHOW_KERNELS = false;
SHOW_UV = true;

%This turns off the feature weighting algorithm, and runs a sum-of-kernels SVM.
AVERAGE_KERNELS = 0;

%NORMALIZATION STUFF
MAX_NORM = 1;
STRIP_DIAG = 1 && ~AVERAGE_KERNELS;
OFF_DIAG = 0 && ~AVERAGE_KERNELS;
CENTERING = 1;


%This sets up an identifying string for saving intermediate results.
N = [AVERAGE_KERNELS MAX_NORM STRIP_DIAG OFF_DIAG CENTERING];

STR = {'', '', '', '', ''; '_AVG', '_max', '_stripd', '_offd', '_center'};
SAVE_STR = '';
for i=1:numel(N), SAVE_STR = strcat(SAVE_STR, STR{logical(N(i))+1,i}); end
for i=1:numel(types), SAVE_STR = strcat(SAVE_STR, '_', types{i}); end
load RUN run_ID;
save_file = sprintf('saved/errs%s_%d.mat', SAVE_STR, run_ID);
alpha_str = sprintf('saved/all_alphas_%d.mat', run_ID);
run_ID = run_ID+1;
save RUN.mat run_ID;
%%%%%%%%%%%%%%%%%%%%%%%

%Basically, 30 x 10
ITER = size(S,1);
error = zeros(ITER, FOLDS);
conf = zeros(ITER, M);
all_alphas = zeros(ITER, FOLDS, num_kernels);

%Parameters
C = 10;
svm_eps = 10^-5;

for I=1:ITER
    Test_Train = floor(1:(FOLDS/M):(FOLDS+1));
    Test_Train = Test_Train(S(I,:))'; %This gets rid of the unwanted extra element.
    
    for f=1:FOLDS
        U = zeros(num_examples, num_kernels);
        V = zeros(num_examples, num_kernels);

        train = Test_Train ~= f;
        TE = numel(find(~train)); % # test subjects per fold. Varies when mod(FOLDS, M) ~= 0.
        TR = numel(find(train));  % # train subjects per fold. Varies when mod(FOLDS, M) ~= 0.

        if AVERAGE_KERNELS == 0
            prev = 1;
            
            for t=1:numel(types)
                K = actually_load(sprintf('kernels_%s/K_%d_%d.mat', types{t}, I, f), 'K');
                K = K(FEATURES{t},KERNELS{t},:,:);
                
                %r is the range of kernels provided by the current
                %modality.
                r = prev:(prev + numel(FEATURES{t})*numel(KERNELS{t}) - 1);
                

                %Compute the within group and between group distances for a particular
                %EXAMPLE in each kernel
                TRP = find(truth(train) == 1);
                TRN = find(truth(train) ~= 1);
                PpE = floor(min(numel(TRP), numel(TRN))/num_examples); %POINTS per EXAMPLE
                EX_ind = zeros(TR,1); %Indicators for which EXAMPLE each POINT belongs to.

                for i=1:num_examples
                    EX_ind(TRP(1:PpE)) = i;
                    TRP(1:PpE) = [];
                    EX_ind(TRN(1:PpE)) = i;
                    TRN(1:PpE) = [];

                    %This is a kernel for just one EXAMPLE
                    K_train = zeros(numel(FEATURES{t})*numel(KERNELS{t}), 2*PpE, 2*PpE);
                    for j=1:size(K,1)
                        for k=1:size(K,2)
                            KM = squeeze(K(j,k,:,:));
                            KM = KM(train,train);

                            if MAX_NORM == 1, KM = KM / max(max(KM)); end
                            if CENTERING == 1, KM = centering(KM); end
                            if OFF_DIAG == 1
                                D = diag(diag(KM));
                                KM = D + (KM - D) ./ sum(sum(KM - D));
                            end
                            if STRIP_DIAG == 1, KM = KM - diag(diag(KM)); end

                            K_train((j-1)*size(K,2)+k,:,:) = KM(EX_ind==i, EX_ind==i);
                        end
                    end %kernels

                    % the kernels, partition
                    TTR = truth(train);
                    [U(i,r), V(i,r)] = create_ratios_frommatrix(K_train, TTR(EX_ind==i));
                end % examples

                prev = r(end)+1;
            end % types

            ALPHA = feature_weigh_convex(U,V);

            all_alphas(I,f,:) = ALPHA;
            save(alpha_str, 'all_alphas');

            if SHOW_UV
                prev = 1;
                for t=1:numel(types)
                    r = prev:(prev + numel(FEATURES{t}) * numel(KERNELS{t}) - 1);
                    figure(t); imagesc(reshape(ALPHA(r), numel(FEATURES{t}), [])); colorbar;
                    title(types{t});
                    prev = r(end)+1;
                end
                pause(1);
            end

        else %AVERAGE_KERNELS
            ALPHA = ones(num_kernels,1);
        end


        %Now we use the subkernel weights ALPHA to assemble training and
        %testing kernels, from which we evaluate their performance in a  single-kernel SVM.
        K_train = zeros(TR,TR);
        K_test = zeros(TR,TE);
        for t=1:numel(types)
            K = actually_load(sprintf('kernels_%s/K_%d_%d.mat', types{t}, I, f), 'K');
            K = K(FEATURES{t}, KERNELS{t}, :,:);

            for j=1:size(K,1)
                for k=1:size(K,2)
                    KM = squeeze(K(j,k,:,:));
                    if MAX_NORM == 1, KM = KM / max(max(KM)); end
                    if CENTERING == 1, KM = centering(KM); end

                    kern_i = (j-1)*size(K,2)+k;
                    K_train = K_train + ALPHA(kern_i) * KM(train,train);
                    K_test = K_test + ALPHA(kern_i) * KM(train,~train);
                end
            end
        end

        %DEBUG STUFF
        if SHOW_KERNELS
            figure(numel(types) + 1);  imagesc(K_train);
            figure(numel(types) + 1);  imagesc(K_test);
            pause;
        end

        %Use Shogun to train the SVM.
        sg('new_svm', 'LIGHT');
        sg('svm_epsilon', svm_eps);
        sg('c', C);
        sg('set_labels', 'TRAIN', truth(train)');
        sg('set_kernel', 'CUSTOM', K_train, 'FULL');
        sg('train_classifier');

        sg('set_kernel', 'CUSTOM', K_test, 'FULL');
        conf(I, ~train) = sg('classify');

        error(I,f) = mean(sign(conf(I, ~train)) ~= truth(~train)');
        disp('#######################################');
        disp(sprintf('Iteration %d Fold %d error is: %f', I, f, error(I,f)));

        save(save_file, 'error', 'conf', 'types', 'FEATURES', 'KERNELS', 'C');
    end %10-fold x-validation
end %30 iterations